@@ -3,7 +3,6 @@ Main entry point for PostgreSQL MCP Server
 """
 
 import asyncio
-import json
 import logging
 import sys
 from typing import Any, Dict, List
@@ -27,14 +26,29 @@ from .resources import (
     get_resource_handlers,
     get_table_schema_resource_handler,
 )
-from mcp import Resource, Tool
+from .protocol_logging import (
+    sanitize_log_output,
+    protocol_logging_server,
+)
+from .prompts import get_prompt_manager
+from mcp import (
+    Resource,
+    Tool,
+    ListPromptsRequest,
+    ListPromptsResult,
+    GetPromptResult,
+)
+import mcp.types as types
 
 
-def setup_logging(log_dir: str = "") -> tuple[logging.Logger, logging.Logger]:
+def setup_logging(
+    log_level: str = "INFO", log_dir: str = ""
+) -> tuple[logging.Logger, logging.Logger]:
     """
-    Setup logging with custom directory
+    Setup logging with custom directory and log level
 
     Args:
+        log_level: Logging level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
         log_dir: Custom log directory path. If empty, uses current directory.
 
     Returns:
@@ -42,6 +56,9 @@ def setup_logging(log_dir: str = "") -> tuple[logging.Logger, logging.Logger]:
     """
     import os
 
+    # ログレベルを数値に変換
+    numeric_level = getattr(logging, log_level.upper(), logging.INFO)
+
     # ログディレクトリが指定されている場合は使用
     if log_dir:
         os.makedirs(log_dir, exist_ok=True)
@@ -56,33 +73,27 @@ def setup_logging(log_dir: str = "") -> tuple[logging.Logger, logging.Logger]:
     for handler in root_logger.handlers[:]:
         root_logger.removeHandler(handler)
 
-    # 基本ログ設定 - ファイルと標準出力の両方に出力
+    # 基本ログ設定 - ファイルのみに出力（sys.stdout/sys.stderrへの出力なし）
     logger = logging.getLogger(__name__)
-    logger.setLevel(logging.DEBUG)
+    logger.setLevel(numeric_level)
 
     # ファイルハンドラー
     file_handler = logging.FileHandler(general_log_path)
-    file_handler.setLevel(logging.DEBUG)
-
-    # 標準出力ハンドラー（フォールバック用）
-    console_handler = logging.StreamHandler(sys.stdout)
-    console_handler.setLevel(logging.INFO)
+    file_handler.setLevel(numeric_level)
 
     # フォーマッター
     formatter = logging.Formatter(
         "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
     )
     file_handler.setFormatter(formatter)
-    console_handler.setFormatter(formatter)
 
-    # ハンドラーを追加
+    # ファイルハンドラーのみ追加
     logger.addHandler(file_handler)
-    logger.addHandler(console_handler)
     logger.propagate = False  # 重複ログを防ぐ
 
     # プロトコルロガー設定
     protocol_logger = logging.getLogger("mcp_protocol")
-    protocol_logger.setLevel(logging.DEBUG)
+    protocol_logger.setLevel(numeric_level)
     protocol_handler = logging.FileHandler(protocol_log_path)
     protocol_handler.setFormatter(
         logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
@@ -101,219 +112,6 @@ protocol_logger = None
 global_config = None
 
 
-def sanitize_log_output(result: Any) -> Any:
-    """
-    ログ出力用に機密情報をマスクする関数
-
-    Args:
-        result: ログ出力する結果データ
-
-    Returns:
-        機密情報がマスクされた結果データ
-    """
-    if isinstance(result, dict):
-        sanitized = result.copy()
-        # 機密情報を含む可能性のあるフィールドをマスク
-        sensitive_fields = ["password", "secret", "token", "key", "auth"]
-        for field in sensitive_fields:
-            if field in sanitized:
-                sanitized[field] = "***MASKED***"
-
-        # ネストされた辞書も再帰的に処理
-        for key, value in sanitized.items():
-            if isinstance(value, dict):
-                sanitized[key] = sanitize_log_output(value)
-            elif isinstance(value, list):
-                sanitized[key] = [
-                    sanitize_log_output(item) if isinstance(item, dict) else item
-                    for item in value
-                ]
-
-        return sanitized
-    elif isinstance(result, list):
-        return [
-            sanitize_log_output(item) if isinstance(item, dict) else item
-            for item in result
-        ]
-    else:
-        return result
-
-
-def sanitize_protocol_message(message: str) -> str:
-    """
-    MCPプロトコルメッセージの機密情報をマスクする関数
-
-    Args:
-        message: JSON形式のプロトコルメッセージ
-
-    Returns:
-        機密情報がマスクされたメッセージ
-    """
-    try:
-        data = json.loads(message)
-        sanitized_data = sanitize_log_output(data)
-        return json.dumps(sanitized_data, ensure_ascii=False)
-    except (json.JSONDecodeError, TypeError):
-        # JSONとして解析できない場合は元のメッセージを返す
-        return message
-
-
-class ProtocolLoggingReceiveStream:
-    """
-    MCPプロトコルメッセージをログに記録する受信ストリームラッパー
-    """
-
-    def __init__(self, original_stream: Any) -> None:
-        self.original_stream = original_stream
-
-        if protocol_logger:
-            protocol_logger.debug(
-                f"PROTOCOL_RECEIVE_STREAM_INIT - original_stream_type: {type(original_stream)}"
-            )
-
-    async def receive(self) -> Any:
-        """受信操作をラップしてログに記録"""
-        try:
-            if protocol_logger:
-                protocol_logger.debug("RECEIVE_START - Waiting for message")
-
-            # 元のストリームのreceiveメソッドを呼び出し
-            data = await self.original_stream.receive()
-
-            if protocol_logger:
-                protocol_logger.debug(
-                    f"RECEIVE_COMPLETE - data_received: {data is not None}, data_type: {type(data)}"
-                )
-
-            if data is not None:
-                try:
-                    if protocol_logger:
-                        # データがbytes型の場合はデコードしてログに記録
-                        if isinstance(data, bytes):
-                            message = data.decode("utf-8").strip()
-                            if message:
-                                sanitized_message = sanitize_protocol_message(message)
-                                protocol_logger.debug(f"REQUEST: {sanitized_message}")
-                            else:
-                                protocol_logger.debug(
-                                    "REQUEST_EMPTY - Empty message received"
-                                )
-                        else:
-                            # その他の型の場合は文字列化してログに記録
-                            message_str = str(data)
-                            sanitized_message = sanitize_protocol_message(message_str)
-                            protocol_logger.debug(f"REQUEST: {sanitized_message}")
-                except Exception as e:
-                    # protocol_loggerがNoneの場合でもエラーを出力しない
-                    if protocol_logger:
-                        protocol_logger.error(f"Error logging request: {e}")
-            return data
-        except Exception as e:
-            # 詳細なエラー情報をログに記録
-            if protocol_logger:
-                import traceback
-
-                protocol_logger.error(
-                    f"RECEIVE_ERROR - error: {e}, traceback: {traceback.format_exc()}"
-                )
-            raise e
-
-    def __getattr__(self, name: str) -> Any:
-        """他のメソッドは元のストリームに委譲"""
-        return getattr(self.original_stream, name)
-
-
-class ProtocolLoggingSendStream:
-    """
-    MCPプロトコルメッセージをログに記録する送信ストリームラッパー
-    """
-
-    def __init__(self, original_stream: Any) -> None:
-        self.original_stream = original_stream
-
-        if protocol_logger:
-            protocol_logger.debug(
-                f"PROTOCOL_SEND_STREAM_INIT - original_stream_type: {type(original_stream)}"
-            )
-
-    async def send(self, item: Any) -> None:
-        """送信操作をラップしてログに記録"""
-        try:
-            if protocol_logger:
-                protocol_logger.debug(
-                    f"SEND_START - item_type: {type(item)}, item_not_none: {item is not None}"
-                )
-
-            if item is not None:
-                try:
-                    if protocol_logger:
-                        # データがbytes型の場合はデコードしてログに記録
-                        if isinstance(item, bytes):
-                            message = item.decode("utf-8").strip()
-                            if message:
-                                sanitized_message = sanitize_protocol_message(message)
-                                protocol_logger.debug(f"RESPONSE: {sanitized_message}")
-                            else:
-                                protocol_logger.debug(
-                                    "RESPONSE_EMPTY - Empty message to send"
-                                )
-                        else:
-                            # その他の型の場合は文字列化してログに記録
-                            message_str = str(item)
-                            sanitized_message = sanitize_protocol_message(message_str)
-                            protocol_logger.debug(f"RESPONSE: {sanitized_message}")
-                except Exception as e:
-                    # protocol_loggerがNoneの場合でもエラーを出力しない
-                    if protocol_logger:
-                        protocol_logger.error(f"Error logging response: {e}")
-
-            # 元のストリームのsendメソッドを呼び出し
-            await self.original_stream.send(item)
-
-            if protocol_logger:
-                protocol_logger.debug("SEND_COMPLETE - Message sent successfully")
-
-        except Exception as e:
-            # 詳細なエラー情報をログに記録
-            if protocol_logger:
-                import traceback
-
-                protocol_logger.error(
-                    f"SEND_ERROR - error: {e}, traceback: {traceback.format_exc()}"
-                )
-            raise e
-
-    def __getattr__(self, name: str) -> Any:
-        """他のメソッドは元のストリームに委譲"""
-        return getattr(self.original_stream, name)
-
-
-async def protocol_logging_server(
-    read_stream: Any, write_stream: Any
-) -> tuple[Any, Any]:
-    """
-    MCPプロトコルメッセージをログに記録するラッパーサーバー
-
-    Args:
-        read_stream: 入力ストリーム
-        write_stream: 出力ストリーム
-
-    Returns:
-        ラップされたストリームのタプル
-    """
-    if protocol_logger:
-        protocol_logger.debug("PROTOCOL_LOGGING_SERVER - Starting stream wrapping")
-
-    # 入出力ストリームをラップ
-    wrapped_read_stream = ProtocolLoggingReceiveStream(read_stream)
-    wrapped_write_stream = ProtocolLoggingSendStream(write_stream)
-
-    if protocol_logger:
-        protocol_logger.debug("PROTOCOL_LOGGING_SERVER - Stream wrapping completed")
-
-    return wrapped_read_stream, wrapped_write_stream
-
-
 async def main() -> None:
     """Main entry point for the MCP server"""
     try:
@@ -321,10 +119,12 @@ async def main() -> None:
         global global_config
         global_config = load_config()
 
-        # ログ設定を再適用（カスタムディレクトリを反映）
+        # ログ設定を再適用（環境変数の設定を反映）
         global logger, protocol_logger
         try:
-            logger, protocol_logger = setup_logging(global_config.log_dir)
+            logger, protocol_logger = setup_logging(
+                log_level=global_config.log_level, log_dir=global_config.log_dir
+            )
             logger.info(f"Configuration loaded successfully. config={global_config}")
         except Exception as log_error:
             # ログ設定失敗時のフォールバック
@@ -333,13 +133,10 @@ async def main() -> None:
                 f"Configuration loaded successfully. config={global_config}",
                 file=sys.stderr,
             )
-            # 最小限のロガーを作成
-            logger = logging.getLogger(__name__)
-            logger.addHandler(logging.StreamHandler(sys.stdout))
-            logger.setLevel(logging.INFO)
-            protocol_logger = logging.getLogger("mcp_protocol")
-            protocol_logger.addHandler(logging.StreamHandler(sys.stdout))
-            protocol_logger.setLevel(logging.INFO)
+            import traceback
+
+            print(f"Server error traceback: {traceback.format_exc()}")
+            sys.exit(1)
 
         # Handle Docker auto-setup if enabled
         if global_config.docker.enabled:
@@ -410,17 +207,48 @@ async def main() -> None:
         if name in all_handlers:
             handler = all_handlers[name]
             try:
+                # プロトコルデバッグモード時の追加ログ
+                if global_config.protocol_debug:
+                    logger.debug(f"TOOL_DEBUG - Executing handler for: {name}")
+                    logger.debug(f"TOOL_DEBUG - Handler function: {handler.__name__}")
+
                 result = await handler(**arguments)
                 # 詳細な出力ログ（機密情報をマスク）
                 sanitized_result = sanitize_log_output(result)
                 logger.info(f"TOOL_OUTPUT - Tool: {name}, Result: {sanitized_result}")
+
+                # プロトコルデバッグモード時の追加ログ
+                if global_config.protocol_debug:
+                    logger.debug(f"TOOL_DEBUG - Raw result type: {type(result)}")
+                    logger.debug(
+                        f"TOOL_DEBUG - Raw result keys: {list(result.keys()) if isinstance(result, dict) else 'N/A'}"
+                    )
+
                 return result
             except Exception as e:
                 logger.error(f"TOOL_ERROR - Tool: {name}, Error: {e}")
+                # プロトコルデバッグモード時の詳細なエラー情報
+                if global_config.protocol_debug:
+                    import traceback
+
+                    logger.debug(
+                        f"TOOL_DEBUG - Error traceback: {traceback.format_exc()}"
+                    )
                 return {"success": False, "error": str(e)}
         else:
             logger.error(f"TOOL_UNKNOWN - Tool: {name}")
-            return {"success": False, "error": f"Unknown tool: {name}"}
+            # JSON-RPC 2.0準拠のエラーレスポンスを返す
+            return {
+                "success": False,
+                "error": {
+                    "code": -32601,
+                    "message": f"Method not found: {name}",
+                    "data": {
+                        "available_methods": list(all_handlers.keys()),
+                        "server_type": "PostgreSQL MCP Server",
+                    },
+                },
+            }
 
     # Register tools via list_tools handler
     @server.list_tools()
@@ -471,6 +299,13 @@ async def main() -> None:
         logger.info(f"RESOURCE_LIST - Total resources available: {total_resources}")
         return resources
 
+    @server.list_resource_templates()
+    async def handle_list_resource_templates() -> list[types.ResourceTemplate]:
+        """List available resource templates"""
+        logger.info("RESOURCE_TEMPLATE_LIST - Listing resource templates")
+        # Currently no resource templates implemented
+        return []
+
     @server.read_resource()
     async def handle_read_resource(uri: str) -> str:
         """Read resource content"""
@@ -513,46 +348,63 @@ async def main() -> None:
         logger.warning(f"RESOURCE_NOT_FOUND - Resource: {uri_str}")
         return f"Resource {uri_str} not found"
 
+    @server.list_prompts()
+    async def handle_list_prompts(request: ListPromptsRequest) -> ListPromptsResult:
+        """List available prompts"""
+        logger.info("PROMPT_LIST - Listing available prompts")
+        try:
+            prompt_manager = get_prompt_manager()
+            prompts = prompt_manager.list_prompts()
+            prompt_count = len(prompts)
+            logger.info(f"PROMPT_LIST_SUCCESS - Found {prompt_count} prompts")
+            return ListPromptsResult(prompts=prompts)
+        except Exception as e:
+            logger.error(f"PROMPT_LIST_ERROR - Error listing prompts: {e}")
+            return ListPromptsResult(prompts=[])
+
+    @server.get_prompt()
+    async def handle_get_prompt(
+        name: str, arguments: dict[str, str] | None
+    ) -> GetPromptResult:
+        """Get prompt content"""
+        logger.info(f"PROMPT_GET - Getting prompt: {name}, arguments: {arguments}")
+        try:
+            prompt_manager = get_prompt_manager()
+            prompt = prompt_manager.get_prompt(name, arguments)
+
+            if prompt:
+                logger.info(f"PROMPT_GET_SUCCESS - Found prompt: {name}")
+                # For now, return empty messages since MCP Prompt doesn't contain message content
+                # In a real implementation, we would need to store the actual message content separately
+                return GetPromptResult(description=prompt.description, messages=[])
+            else:
+                logger.warning(f"PROMPT_NOT_FOUND - Prompt not found: {name}")
+                return GetPromptResult(description="", messages=[])
+        except Exception as e:
+            logger.error(f"PROMPT_GET_ERROR - Error getting prompt {name}: {e}")
+            return GetPromptResult(description="", messages=[])
+
     # Start the server
     logger.info("Starting PostgreSQL MCP Server...")
-    protocol_logger.info("MCP Protocol logging started")
 
     try:
         async with stdio_server() as (read_stream, write_stream):
-            if protocol_logger:
-                protocol_logger.debug(
-                    "STDIO_SERVER - stdio_server context entered successfully"
-                )
-
             # プロトコルロギングを有効化
             read_stream, write_stream = await protocol_logging_server(
-                read_stream, write_stream
+                read_stream, write_stream, global_config, protocol_logger
             )
 
-            if protocol_logger:
-                protocol_logger.debug("SERVER_RUN - Starting server.run()")
-
             await server.run(
                 read_stream, write_stream, server.create_initialization_options()
             )
 
-            if protocol_logger:
-                protocol_logger.debug("SERVER_RUN - server.run() completed normally")
-
     except Exception as e:
-        if protocol_logger:
-            import traceback
-
-            protocol_logger.error(
-                f"SERVER_ERROR - Exception in main server loop: {e}, traceback: {traceback.format_exc()}"
-            )
         logger.error(f"Server error: {e}")
         import traceback
 
         logger.error(f"Server error traceback: {traceback.format_exc()}")
-        # 詳細なエラー情報を標準エラー出力にも出力
-        print(f"Server error: {e}", file=sys.stderr)
-        print(f"Server error traceback: {traceback.format_exc()}", file=sys.stderr)
+        # 詳細なエラー情報はファイルにのみ出力（sys.stderr/sys.stdoutへの出力なし）
+        # 既にlogger.errorでファイルに出力されているため、追加の出力は不要
         raise
 
 
